@@ -21,6 +21,7 @@
 #include <qpdf/BufferInputSource.hh>
 #include <qpdf/QPDFWriter.hh>
 #include <qpdf/QPDFPageDocumentHelper.hh>
+#include <qpdf/Pl_Discard.hh>
 
 #include <pybind11/stl.h>
 #include <pybind11/iostream.h>
@@ -41,6 +42,12 @@ void check_stream_is_usable(py::object stream)
     }
 }
 
+void qpdf_basic_settings(QPDF& q)
+{
+    q.setSuppressWarnings(true);
+    q.setImmediateCopyFrom(true);
+}
+
 std::shared_ptr<QPDF>
 open_pdf(
     py::object filename_or_stream,
@@ -53,37 +60,39 @@ open_pdf(
 {
     auto q = std::make_shared<QPDF>();
 
+    qpdf_basic_settings(*q);
     q->setSuppressWarnings(suppress_warnings);
     q->setPasswordIsHexKey(hex_password);
     q->setIgnoreXRefStreams(ignore_xref_streams);
     q->setAttemptRecovery(attempt_recovery);
-    q->setImmediateCopyFrom(true);
+
+    py::object stream;
+    bool closing_stream;
+    std::string description;
 
     if (py::hasattr(filename_or_stream, "read") && py::hasattr(filename_or_stream, "seek")) {
         // Python code gave us an object with a stream interface
-        py::object stream = filename_or_stream;
-
+        stream = filename_or_stream;
         check_stream_is_usable(stream);
+        closing_stream = false;
+        description = py::repr(stream);
+    } else {
+        if (py::isinstance<py::int_>(filename_or_stream))
+            throw py::type_error("expected str, bytes or os.PathLike object");
+        auto filename = fspath(filename_or_stream);
+        auto io_open = py::module::import("io").attr("open");
+        stream = io_open(filename, "rb");
+        closing_stream = true;
+        description = py::str(filename);
+    }
 
-        // The PythonInputSource object will be owned by q
-        auto input_source = PointerHolder<InputSource>(new PythonInputSource(stream));
+    auto input_source = PointerHolder<InputSource>(new PythonInputSource(
+        stream, description, closing_stream
+    ));
+
+    {
         py::gil_scoped_release release;
         q->processInputSource(input_source, password.c_str());
-    } else {
-        auto filename = filename_or_stream;
-        std::string description = py::str(filename);
-        FILE* file = portable_fopen(filename_or_stream, "rb");
-
-        // We can release GIL because Python knows nothing about q at this
-        // point; this could also take a moment for large files
-        py::gil_scoped_release release;
-        q->processFile(
-            description.c_str(),
-            file, // transferring ownership
-            true, // QPDF will close the file (including if there are exceptions)
-            password.c_str()
-        );
-        file = nullptr; // QPDF owns the file and will close it
     }
 
     if (inherit_page_attributes) {
@@ -369,6 +378,8 @@ void save_pdf(
         check_stream_is_usable(stream);
         description = py::repr(stream);
     } else {
+        if (py::isinstance<py::int_>(filename_or_stream))
+            throw py::type_error("expected str, bytes or os.PathLike object");
         py::object filename = fspath(filename_or_stream);
         py::object ospath = py::module::import("os").attr("path");
         py::object samefile = ospath.attr("samefile");
@@ -460,7 +471,7 @@ void init_qpdf(py::module &m)
             []() {
                 auto q = std::make_shared<QPDF>();
                 q->emptyPDF();
-                q->setSuppressWarnings(true);
+                qpdf_basic_settings(*q);
                 return q;
             },
             "Create a new empty PDF from stratch."
@@ -583,7 +594,7 @@ void init_qpdf(py::module &m)
             Return type:
                 pikepdf._qpdf.PageList
             )~~~",
-            py::keep_alive<0, 1>()
+            py::return_value_policy::reference_internal
         )
         .def_property_readonly("_pages", &QPDF::getAllPages)
         .def_property_readonly("is_encrypted", &QPDF::isEncrypted)
@@ -806,6 +817,21 @@ void init_qpdf(py::module &m)
             py::arg("objid"),
             py::arg("gen")
         )
+        .def_property_readonly("objects",
+            [](QPDF &q) {
+                return q.getAllObjects();
+            },
+            R"~~~(
+            Return an iterable list of all objects in the PDF.
+
+            After deleting content from a PDF such as pages, objects related
+            to that page, such as images on the page, may still be present.
+
+            Retun type:
+                pikepdf._ObjectList
+            )~~~",
+            py::return_value_policy::reference_internal
+        )
         .def("make_indirect", &QPDF::makeIndirectObject,
             R"~~~(
             Attach an object to the Pdf as an indirect object
@@ -877,6 +903,15 @@ void init_qpdf(py::module &m)
             Used to implement Pdf.close().
             )~~~"
         )
+        .def("_decode_all_streams_and_discard",
+            [](QPDF &q) {
+                QPDFWriter w(q);
+                Pl_Discard discard;
+                w.setOutputPipeline(&discard);
+                w.setDecodeLevel(qpdf_dl_all);
+                w.write();
+            }
+        )
         .def_property_readonly("_allow_accessibility",
             [](QPDF &q) {
                 return q.allowAccessibility();
