@@ -5,7 +5,7 @@ from sys import getrefcount as refcount
 
 import pytest
 
-from pikepdf import Pdf, PdfMatrix, Stream
+from pikepdf import Pdf, PdfMatrix, Stream, Dictionary, Name
 
 
 # pylint: disable=redefined-outer-name,pointless-statement
@@ -232,6 +232,8 @@ def test_bad_insert(fourpages):
     pdf = fourpages
     with pytest.raises(TypeError):
         pdf.pages.insert(0, 'this is a string not a page')
+    with pytest.raises(TypeError):
+        pdf.pages.insert(0, Dictionary(Type=Name.NotAPage, Value="Not a page"))
 
 
 def test_negative_indexing(fourpages, graph):
@@ -250,7 +252,6 @@ def test_negative_indexing(fourpages, graph):
 def test_concatenate(resources, outdir):
     # Issue #22
     def concatenate(n):
-        print('concatenating same page', n, 'times')
         output_pdf = Pdf.new()
         for i in range(n):
             print(i)
@@ -268,3 +269,47 @@ def test_emplace(fourpages):
     assert fourpages.pages[0].keys() == fourpages.pages[1].keys()
     for k in fourpages.pages[0].keys():
         assert fourpages.pages[0][k] == fourpages.pages[1][k]
+
+
+def test_duplicate_page(sandwich, outpdf):
+    sandwich.pages.append(sandwich.pages[0])
+    assert len(sandwich.pages) == 2
+    sandwich.save(outpdf)
+
+
+def test_repeat_using_intermediate(graph, outpdf):
+    def _repeat_page(pdf_in, page, count, pdf_out):
+        for dup in range(count):
+            pdf_new = Pdf.new()
+            pdf_new.pages.append(pdf_in.pages[page])
+            pdf_out.pages.extend(pdf_new.pages)
+        return pdf_out
+
+    with Pdf.new() as out:
+        _repeat_page(graph, 0, 3, out)
+        assert len(out.pages) == 3
+        out.save(outpdf)
+
+
+def test_repeat(graph, outpdf):
+    def _repeat_page(pdf, page, count):
+        for dup in range(count):
+            pdf.pages.append(pdf.pages[page])
+        return pdf
+
+    _repeat_page(graph, 0, 3)
+    assert len(graph.pages) == 4
+    graph.save(outpdf)
+
+
+def test_add_foreign_twice(graph, outpdf):
+    out = Pdf.new()
+    out.pages.append(out.copy_foreign(graph.pages[0]))
+    assert len(out.pages) == 1
+    out.pages.append(out.copy_foreign(graph.pages[0]))
+    assert len(out.pages) == 2
+    out.save(outpdf)
+
+
+def test_repr_pagelist(fourpages):
+    assert '4' in repr(fourpages.pages)
